from re import findall

import pytest

from .unous_requests import UnousRequests


@pytest.mark.parametrize('method', ['GET', 'POST'])
def test_requisicao_deve_ser_efetuada_corretamente(
    method,
    requisicao_get_positiva_mockada,
    requisicao_post_positiva_mockada,
):
    ur = UnousRequests()
    request_method = getattr(ur, method.lower())
    response = request_method('http://requestb.in')
    assert response.ok is True


@pytest.mark.parametrize('method', ['GET', 'POST'])
def test_log_deve_ser_exibido_se_requisicao_gerar_excecao(
    method,
    capsys,
    requisicao_get_negativa_mockada,
    requisicao_post_negativa_mockada,
):
    ur = UnousRequests()
    method = getattr(ur, method.lower())
    method('http://requestb.in', timeout=000.1)
    read_timeout = capsys.readouterr()
    regex = r'Um\s\w+\socorreu\sao\sexecutar\sum\s(?:GET|POST)\sna\surl\shtt(?:ps|p):\W+\w+.\w+'
    assert findall(regex, read_timeout.out)


@pytest.mark.parametrize('method', ['GET', 'POST'])
def test_requisicao_get_deve_falhar_se_o_tempo_resposta_exceder(
    method,
    requisicao_get_negativa_mockada,
    requisicao_post_negativa_mockada,
):
    ur = UnousRequests()
    method = getattr(ur, method.lower())
    response = method('http://requestb.in', timeout=000.1)
    assert not response.ok





import decimal
import json
import os
import time

from .unous_requests import UnousRequests


class ApiUnous(UnousRequests):
    _grant_type = 'password'
    _client_id = 'userIntegration'
    _mindset_user = os.environ.get('MINDSET_USER')
    _mindset_pass = os.environ.get('MINDSET_PASS')
    _mindset_url = os.environ.get('MINDSET_URL')
    _mindset_notify_url = os.environ.get('MINDSET_NOTIFY_URL')
    show_log = True

    @property
    def headers(self):
        return {
            'Content-Type': 'application/json',
            'Authorization': f'Bearer {self._get_token()}',
        }

    def integrar_produtos(self, dados):
        return self._integrar(dados, '/StageContent/api/Product/Post')

    def integrar_produtos_tamanhos(self, dados):
        return self._integrar(dados, '/StageContent/api/ProductSize/Post')

    def integrar_fornecedores(self, dados):
        return self._integrar(dados, '/StageContent/api/Supplier/Post')

    def integrar_pedidos(self, dados):
        return self._integrar(dados, '/StageMetrics/api/OpenOrder/Post')

    def integrar_lojas(self, dados):
        return self._integrar(dados, '/StageContent/api/Location/Post')

    def integrar_lojas_info(self, dados):
        return self._integrar(dados, '/StageContent/api/StoreInfo/Post')

    def integrar_metricas(self, dados):
        return self._integrar(dados, '/StageMetrics/api/Metric/Post')

    def integrar_pedidos_sku(self, dados):
        return self._integrar(dados, '/StageAllocation/api/OrderSkuData/Post')

    def integrar_cluster_pedidos_sku(self, dados):
        return self._integrar(dados, '/StageAllocation/api/OrderClusterSkuData/Post')

    def integrar_grade_pedidos_sku(self, dados):
        return self._integrar(dados, '/StageAllocation/api/GridAllocationModelOpenOrderDcSku/Post')

    def integrar_grade_disponiveis_cd(self, dados):
        return self._integrar(dados, '/StageAllocation/api/GridAllocationModelDcSku/Post')

    def integrar_sku_disponiveis_cd(self, dados):
        return self._integrar(dados, '/StageAllocation/api/DCPositionSKUAlloc/Post')

    def integrar_pedidos_packs_serem_alocados(self, dados):
        return self._integrar(dados, '/StageAllocation/api/OrderPackData/Post')

    def integrar_cluster_pedidos_packs_serem_alocados(self, dados):
        return self._integrar(dados, '/StageAllocation/api/OrderPackClusterData/Post')

    def integrar_produtos_atendimento_cd(self, dados):
        return self._integrar(dados, '/StageAllocation/api/OrderInDistribution/Post')

    def integrar_grade_pedidos_pack(self, dados):
        return self._integrar(dados, '/StageAllocation/api/GridAllocationModelOpenOrderPack/Post')

    def integrar_posicao_estoque_sku(self, dados):
        return self._integrar(dados, '/StageAllocation/api/DCPositionSKUAlloc/Post')

    def notificar(self):
        params = {
            'EnterpriseKey': self._mindset_user,
            'enterprisePwd': self._mindset_pass,
            'jobName': 'CleanUpJob',
        }
        response = self.get(url=self._mindset_notify_url, params=params)
        return {
            'ok': response.ok,
            'notificou': response.json().get('StatusReply', '') == 0,
        }

    def _integrar(self, dados, url_endpoint):
        url, total_registros = self._mindset_url + url_endpoint, 0
        for lote in self.cria_lotes(dados):
            lote_enviado = False
            while not lote_enviado:
                retorno = self.posta_lote_na_unous(url=url, lote=lote)
                print(f"Status_code: {retorno.status_code}")
                if not retorno.ok:
                    print(f"Retorno: {retorno}")
                    time.sleep(5)
                else:
                    total_registros += len(lote)
                    lote_enviado = True
        print(f'{total_registros} REGISTROS INTEGRADOS')
        print('-'*42)

    def limpar_pedidos(self):
        url = self._mindset_url + '/StageMetrics/api/OpenOrder/GetClearAllData'
        self.get(url=url, headers=self.headers)

    def _get_token(self):
        response = self.get(
            url=self._mindset_url + '/Auth/token',
            data={
                'grant_type': self._grant_type,
                'username': self._mindset_user,
                'password': self._mindset_pass,
                'client_id': self._client_id,
            }
        )
        if not response.ok:
            raise ValueError(f'Erro ao obter o token de acesso! Status: {response.status_code}')
        return json.loads(response.text).get('access_token')

    def posta_lote_na_unous(self, url, lote):
        return self.post(url=url, data=json.dumps(lote, cls=DecimalEncoder), headers=self.headers)

    def cria_lotes(self, dados, tamanho=int(os.environ.get('TAMANHO_LOTE_PARA_POSTAGEM_UNOUS', 10000))):
        for i in range(0, len(dados), tamanho):
            yield dados[i:i + tamanho]


class DecimalEncoder(json.JSONEncoder):

    def default(self, o):
        if isinstance(o, decimal.Decimal):
            return float(o)
        return super(DecimalEncoder, self).default(o)



from datetime import datetime
from json import dumps
from os import environ

import requests
from pytz import timezone
from requests.models import Response


class UnousRequests:

    def __init__(self, show_log=True):
        self.show_log = show_log

    def get(
        self,
        url,
        headers={},
        params=None,
        data={},
        json=None,
        timeout=int(environ.get('TIMEOUT_REQUISICAO_UNOUS', 10)),
    ):
        try:
            return requests.get(
                url=url,
                headers=headers,
                params=params,
                json=json,
                data=data,
                timeout=timeout,
            )
        except Exception as e:
            return self.create_error_response(e, 'GET', url)

    def post(
        self,
        url,
        headers={},
        params=None,
        data={},
        json=None,
        timeout=int(environ.get('TIMEOUT_REQUISICAO_UNOUS', 10)),
    ):
        try:
            return requests.post(
                url=url,
                headers=headers,
                params=params,
                data=data,
                json=json,
                timeout=timeout,
            )
        except Exception as e:
            return self.create_error_response(e, 'POST', url)

    def create_error_response(self, error, method, url):
        erro, agora = error.__class__.__name__, datetime.now(tz=timezone('America/Sao_Paulo'))
        log = f'{agora}: Um {erro} ocorreu ao executar um {method} na url {url}.'
        response = Response()
        response.encoding = 'utf-8'
        response.status_code = 408
        response.url = url
        response._content = dumps({'erro': log[34:]}).encode('utf-8')
        print(log) if self.show_log else None
        return response



from random import randint

import pytest


def test_instancia_api_unous_deve_notificar_url_clean_up(
    api_unous,
    requisicao_get_positiva_mockada,
    resposta_positiva_requisicao,
    unous_token,
    mocker,
):
    resposta_positiva_requisicao._content = '{"StatusReply": 0}'.encode('UTF-8')
    mocker.patch('lins_unous.lins_unous.ApiUnous.get', return_value=resposta_positiva_requisicao)
    response = api_unous.notificar()
    saida_esperada = {'ok': True, 'notificou': True}
    assert response == saida_esperada


@pytest.mark.parametrize(
    'integracao',
    [
        'integrar_produtos',
        'integrar_produtos_tamanhos',
        'integrar_fornecedores',
        'integrar_pedidos',
        'integrar_lojas',
        'integrar_lojas_info',
        'integrar_metricas',
    ],
)
def test_integracao_deve_ser_efetuada_corretamente(
    integracao,
    api_unous,
    requisicao_post_positiva_mockada,
    capsys,
    mocked_sleep,
):
    dados = [{}] * randint(10000, 100000)
    integracao = getattr(api_unous, integracao)
    integracao(dados)
    log = capsys.readouterr()
    requisicao_post_positiva_mockada.assert_called()
    assert f'{len(dados)} REGISTROS INTEGRADOS' in log.out
